@@ -76,7 +76,6 @@ def eval(state, model, objective, gen):
 
 
 def main(**kw_args):
-
     # Setup arguments.
     parser = argparse.ArgumentParser()
     parser.add_argument("--root", type=str, nargs="*", default=["_experiments"])
@@ -139,7 +138,7 @@ def main(**kw_args):
     parser.add_argument("--evaluate", action="store_true")
     parser.add_argument("--evaluate-last", action="store_true")
     parser.add_argument("--evaluate-fast", action="store_true")
-    parser.add_argument("--evaluate-num-plots", type=int, default=0)
+    parser.add_argument("--evaluate-num-plots", type=int, default=5)
     parser.add_argument(
         "--evaluate-objective",
         choices=["loglik", "elbo"],
@@ -153,6 +152,12 @@ def main(**kw_args):
     parser.add_argument("--also-ar", action="store_true")
     parser.add_argument("--no-ar", action="store_true")
     parser.add_argument("--experiment-setting", type=str, nargs="*")
+    parser.add_argument(
+        "--eeg-mode",
+        type=str,
+        choices=["random", "interpolation", "forecasting", "reconstruction"],
+    )
+    parser.add_argument("--patch", type=str)
 
     if kw_args:
         # Load the arguments from the keyword arguments passed to the function.
@@ -169,6 +174,37 @@ def main(**kw_args):
     else:
         args = parser.parse_args()
 
+    def patch_model(d):
+        """Patch a loaded model.
+
+        Args:
+            d (dict): Output of :func:`torch.load`.
+
+        Returns:
+            dict: `d`, but patched.
+        """
+        if args.patch:
+            with out.Section("Patching loaded model"):
+                # Loop over patches.
+                for patch in args.patch.strip().split(";"):
+                    base_from, base_to = patch.split(":")
+
+                    # Try to apply the patch.
+                    applied_patch = False
+                    for k in list(d["weights"].keys()):
+                        if k.startswith(base_from):
+                            applied_patch = True
+                            tail = k[len(base_from) :]
+                            d["weights"][base_to + tail] = d["weights"][k]
+                            del d["weights"][k]
+
+                    # Report whether the patch was applied.
+                    if applied_patch:
+                        out.out(f'Applied patch "{patch}".')
+                    else:
+                        out.out(f'Did not apply patch "{patch}".')
+        return d
+
     # Remove the architecture argument if a model doesn't use it.
     if args.model not in {
         "convcnp",
@@ -201,13 +237,16 @@ def main(**kw_args):
         # The default is training.
         suffix = "_train"
 
+    data_dir = args.data if args.mean_diff is None else f"{args.data}-{args.mean_diff}"
+    data_dir = data_dir if args.eeg_mode is None else f"{args.data}-{args.eeg_mode}"
+
     # Setup script.
     if not observe:
         out.report_time = True
     wd = WorkingDirectory(
         *args.root,
         *(args.subdir or ()),
-        args.data if args.mean_diff is None else f"{args.data}-{args.mean_diff}",
+        data_dir,
         *((f"x{args.dim_x}_y{args.dim_y}",) if hasattr(args, "dim_x") else ()),
         args.model,
         *((args.arch,) if hasattr(args, "arch") else ()),
@@ -258,6 +297,7 @@ def main(**kw_args):
         # doesn't make sense to set it to a value higher of the last hidden layer of
         # the CNN architecture. We therefore set it to 64.
         "num_basis_functions": 64,
+        "eeg_mode": args.eeg_mode,
     }
 
     # Setup data generators for training and for evaluation.
@@ -279,7 +319,7 @@ def main(**kw_args):
     # Check if a run has completed.
     if args.check_completed:
         if os.path.exists(wd.file("model-last.torch")):
-            d = torch.load(wd.file("model-last.torch"), map_location="cpu")
+            d = patch_model(torch.load(wd.file("model-last.torch"), map_location="cpu"))
             if d["epoch"] >= args.epochs - 1:
                 out.out("Completed!")
                 sys.exit(0)
@@ -558,7 +598,9 @@ def main(**kw_args):
             name = "model-last.torch"
         else:
             name = "model-best.torch"
-        model.load_state_dict(torch.load(wd.file(name), map_location=device)["weights"])
+        model.load_state_dict(
+            patch_model(torch.load(wd.file(name), map_location=device))["weights"]
+        )
 
         if not args.ar or args.also_ar:
             # Make some plots.
@@ -615,8 +657,12 @@ def main(**kw_args):
         start = 0
         if args.resume_at_epoch:
             start = args.resume_at_epoch - 1
-            d_last = torch.load(wd.file("model-last.torch"), map_location=device)
-            d_best = torch.load(wd.file("model-best.torch"), map_location=device)
+            d_last = patch_model(
+                torch.load(wd.file("model-last.torch"), map_location=device)
+            )
+            d_best = patch_model(
+                torch.load(wd.file("model-best.torch"), map_location=device)
+            )
             model.load_state_dict(d_last["weights"])
             best_eval_lik = d_best["objective"]
         else:
